<?php

namespace Drupal\cforge\EventSubscriber;

use Symfony\Component\EventDispatcher\EventSubscriberInterface;

/**
 * Migration modifications
 */
class MigrationSubscriber implements EventSubscriberInterface {

  const FILTER_FORMATS = [
    'editor_full_html' => 'full_html',
    'editor_filtered_html' => 'basic',
    'filtered_html' => 'basic',
    'full_html' => 'full_html',
    'plain_text' => ''
  ];

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents() {
    return [
       //AKA Drupal\migrate\Event\MigrateEvents::PRE_IMPORT
       'migrate.pre_row_save' => [['migratePreRowSave']]
    ];
  }

  /**
   * @param Drupal\migrate\Event\MigratePreRowSave $event
   */
  public function migratePreRowSave($event) {
    $row = $event->getRow();
    $migration = $event->getMigration();

    if($row->getSourceProperty('plugin') == 'd7_user') {
      // Rename the fields
      $map = [
        'manage_notes' => 'notes_admin',
        'profile_notes' => 'notes',//this isn't on the WestieLETS source
        'profile_phones' => 'phones',
        'profile_address' => 'address'
      ];
      foreach ($map as $old_name => $new_name) {
        if ($row->hasDestinationProperty($old_name)) {
          $row->setDestinationProperty($new_name, $row->getDestinationProperty($old_name));
          $row->removeDestinationProperty($old_name);
        }
      }
    }
    // Move all terms that used to be in offers_wants_categories field.
    if($migration->id() == 'd7_node' or $migration->id() == 'd7_mcapi_transaction') {
      if ($row->hasDestinationProperty('offers_wants_categories')) {
        $entity_type = $row->getSourceProperty('entity_type');
        $new_field_name = $row->getSourceProperty('entity_type') == 'node' ? 'categories' : 'category';
        $row->setDestinationProperty($new_field_name, $row->getSourceProperty('offers_wants_categories'));
        $row->removeDestinationProperty('offers_wants_categories');
      }
    }
    // Change the filter format for every body field
    // I don't know why this doesn't happen already
    if($row->getSourceProperty('plugin') == 'd7_node') { //what about d7_comment?
      $this->entityBodyFilterFormat($row, 'body');

      // Change the name of the image taxonomy field
      if ($migration->id() == 'd7_node:image') {
        $row->setDestinationProperty('gallery', $row->getDestinationProperty('galleries'));
        $row->removeDestinationProperty('galleries');
      }
      // Change the name of the document taxonomy field
      if ($migration->id() == 'd7_node:document') {
        $row->setDestinationProperty('doctype', $row->getDestinationProperty('cforge_docs_categories'));
        $row->removeDestinationProperty('cforge_docs_categories');
      }
    }

    if ($migration->id() == 'd7_taxonomy_vocabulary') {
      if (in_array($row->getSourceProperty('machine_name'), ['galleries', 'cforge_docs_categories'])) {
        throw new MigrateSkipRowException();
      }
    }
    // Move the terms to the new gallery name
    if ($migration->id() == 'd7_taxonomy_term:galleries') {
      $row->setDestinationProperty('vid', 'galleries');
    }
    elseif ($migration->id() == 'd7_taxonomy_term:cforge_docs_categories') {
      $row->setDestinationProperty('vid', 'binders');
    }

    if($migration->id() == 'd7_comment' and $row->getDestinationProperty('entity_type') != 'smallad') {
      $row->setDestinationProperty('comment_type', $row->getDestinationProperty('entity_type'));
      $row->setDestinationProperty('field_name', 'comments');
      $row->setDestinationProperty('body', $row->getDestinationProperty('comment_body'));// Assumes no comments on transactions
    }

    // Move the terms to the new gallery name
    if ($migration->id() == 'contact_category') {
     echo "\nChecking contact import:"; print_r($row->getDestination());
    }
  }

  /**
   *
   * @param Row $row
   * @param string $body_field_name
   *
   * @deprecated
   */
  private function entityBodyFilterFormat($row, $body_field_name) {
    if ($bods = $row->getDestinationProperty($body_field_name)) {
      $map = static::FILTER_FORMATS;
      foreach ($bods as $delta => &$body) {
        if ($new_format = $map[$body['format']]) {
          $bods[$delta]['format'] = $new_format;
        }
      }
      $row->setDestinationProperty($body_field_name, $bods);
    }
  }

}
